/**
 * FingerprintJS Botd v0.1.24 - Copyright (c) FingerprintJS, Inc, 2022 (https://fingerprintjs.com)
 * Licensed under the MIT (http://www.opensource.org/licenses/mit-license.php) license.
 */

'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

function __awaiter(thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
}

/**
 * Represents the bot detection error.
 */
class BotdError extends Error {
    /**
     * Creates a new BotdError.
     *
     * @class
     */
    constructor(state, message) {
        super(message);
        this.state = state;
        this.name = 'BotdError';
    }
}

function getUserAgent() {
    return navigator.userAgent;
}

function hasUserAgentData() {
    return navigator.userAgentData !== undefined;
}

function getAppVersion() {
    return navigator.appVersion;
}

function getRTT() {
    if (navigator.connection === undefined) {
        throw new BotdError(-1 /* Undefined */, 'navigator.connection is undefined');
    }
    return navigator.connection.rtt;
}

function getWindowOuterSize() {
    return [window.outerWidth, window.outerHeight];
}

function arePermissionsInconsistent() {
    return __awaiter(this, void 0, void 0, function* () {
        if (window.Notification === undefined) {
            throw new BotdError(-1 /* Undefined */, 'window.Notification is undefined');
        }
        if (navigator.permissions === undefined) {
            throw new BotdError(-1 /* Undefined */, 'navigator.permissions is undefined');
        }
        const { permissions } = navigator;
        if (typeof permissions.query !== 'function') {
            throw new BotdError(104 /* NotFunction */, 'navigator.permissions.query is not a function');
        }
        const permissionStatus = yield permissions.query({ name: 'notifications' });
        return window.Notification.permission === 'denied' && permissionStatus.state === 'prompt';
    });
}

function getInstallTrigger() {
    return window.InstallTrigger !== undefined;
}

function getWebGL() {
    const canvasElement = document.createElement('canvas');
    if (typeof canvasElement.getContext !== 'function') {
        throw new BotdError(104 /* NotFunction */, 'HTMLCanvasElement.getContext is not a function');
    }
    const webGLContext = canvasElement.getContext('webgl');
    if (webGLContext === null) {
        throw new BotdError(101 /* Null */, 'WebGLRenderingContext is null');
    }
    else {
        if (typeof webGLContext.getParameter !== 'function') {
            throw new BotdError(104 /* NotFunction */, 'WebGLRenderingContext.getParameter is not a function');
        }
        const vendor = webGLContext.getParameter(webGLContext.VENDOR);
        const renderer = webGLContext.getParameter(webGLContext.RENDERER);
        const version = webGLContext.getParameter(webGLContext.VERSION);
        if (getInstallTrigger()) {
            return [vendor, renderer, version];
        }
        else {
            const webGLDebugInfo = webGLContext.getExtension('WEBGL_debug_renderer_info');
            if (webGLDebugInfo === null) {
                throw new BotdError(101 /* Null */, 'WEBGL_debug_renderer_info extension is null');
            }
            else {
                const unmaskedVendor = webGLContext.getParameter(webGLDebugInfo.UNMASKED_VENDOR_WEBGL);
                const unmaskedRenderer = webGLContext.getParameter(webGLDebugInfo.UNMASKED_RENDERER_WEBGL);
                return [unmaskedVendor, unmaskedRenderer, vendor, renderer, version];
            }
        }
    }
}

function getScreen() {
    return [screen.width, screen.height];
}

function getDeviceMemory() {
    if (navigator.deviceMemory === undefined) {
        throw new BotdError(-1 /* Undefined */, 'navigator.deviceMemory is undefined');
    }
    return navigator.deviceMemory;
}

function isBigEndian() {
    const buf = new ArrayBuffer(4);
    new Uint32Array(buf)[0] = 0xaa000000;
    return new Uint8Array(buf)[0] === 0xaa;
}

function getHardwareConcurrency() {
    try {
        const concurrency = navigator.hardwareConcurrency;
        if (typeof concurrency === 'string') {
            const concurrencyInt = parseInt(concurrency);
            return isNaN(concurrencyInt) ? 1 : concurrencyInt;
        }
        return concurrency;
    }
    catch (e) {
        throw new BotdError(103 /* WrongType */, 'navigator.hardwareConcurrency wrong type');
    }
}

function getEvalLength() {
    return eval.toString().length;
}

function getPluginsLength() {
    if (navigator.plugins === undefined) {
        throw new BotdError(-1 /* Undefined */, 'navigator.plugins is undefined');
    }
    return navigator.plugins.length;
}

function arePluginsConsistent() {
    return __awaiter(this, void 0, void 0, function* () {
        if (navigator.plugins === undefined) {
            return false;
        }
        const { plugins } = navigator;
        let isConsistent = Object.getPrototypeOf(plugins) === PluginArray.prototype;
        for (let i = 0; i < plugins.length; i++) {
            isConsistent && (isConsistent = Object.getPrototypeOf(plugins[i]) === Plugin.prototype);
        }
        return isConsistent;
    });
}

function getErrorTrace() {
    try {
        // eslint-disable-next-line @typescript-eslint/ban-ts-comment
        // @ts-ignore
        null[0]();
    }
    catch (error) {
        return error.stack.toString();
    }
    throw new BotdError(102 /* UnexpectedBehaviour */, 'errorTrace signal unexpected behaviour');
}

function getErrorFF() {
    try {
        throw 'a';
    }
    catch (e) {
        try {
            e.toSource();
            return true;
        }
        catch (e2) {
            return false;
        }
    }
}

function getOSCPU() {
    if (navigator.oscpu === undefined) {
        throw new BotdError(-1 /* Undefined */, 'navigator.oscpu is undefined');
    }
    return navigator.oscpu;
}

function getPlatform() {
    return navigator.platform;
}

function getProductSub() {
    return navigator.productSub;
}

function getVendor() {
    return navigator.vendor;
}

function getWebDriver() {
    if (navigator.webdriver == undefined) {
        throw new BotdError(-1 /* Undefined */, 'navigator.webdriver is undefined');
    }
    return navigator.webdriver;
}

function getFrequency() {
    function now() {
        return performance.now() / 1000;
    }
    function gcd(a, b) {
        if (a < 0.00000001) {
            return b;
        }
        if (a < b) {
            return gcd(b - Math.floor(b / a) * a, a);
        }
        else if (a === b) {
            return a;
        }
        else {
            return gcd(b, a);
        }
    }
    const x = now();
    let g = now() - x;
    for (let i = 0; i < 10; i++) {
        g = gcd(g, now() - x);
    }
    return Math.round(1 / g);
}

function getProperties(object, properties = []) {
    if (Object.getPrototypeOf(object) == null)
        return properties;
    return getProperties(Object.getPrototypeOf(object), properties.concat(Object.getOwnPropertyNames(object)));
}
const re = /(^(.{0,5})$)|(^.*(-|_|\$|[jJ][sS]|[uU]ser|[sS]cript|[iI]nit|[dD]river|[aA]uto|[aA]gent|[sS]end|[lL]oad|[dD]ev|[cC]all|[bB]..f|[pP]rint|[kK]it|ium|[aA]rray|[pP]romise|[sS]ymbol|[cC]reate|[cC]onst).*$)|(^([A-Z_])*$)|(^([a-z-]){0,9}$)/;

function getWindowProperties() {
    let raw = getProperties(window);
    const chromeIndex = raw.indexOf('chrome');
    const safariIndex = raw.indexOf('safari');
    raw = raw.filter((el, i) => {
        const matchRe = el.match(re) != null;
        const chromeAround = chromeIndex != -1 ? i > chromeIndex - 5 && i < chromeIndex + 5 : false;
        const safariAround = safariIndex != -1 ? i > safariIndex - 5 && i < safariIndex + 5 : false;
        return matchRe || chromeAround || safariAround;
    });
    return raw;
}

function getDocumentProperties() {
    const raw = getProperties(window.document);
    return raw.filter((el) => el.match(re) != null);
}

function getNavigatorProperties() {
    return getProperties(window.navigator);
}

function getTouchPoints() {
    return [navigator.maxTouchPoints, navigator.msMaxTouchPoints];
}

function getSourceBufferType() {
    return [typeof SourceBuffer, typeof SourceBufferList];
}

function getDocumentElementKeys() {
    if (document.documentElement === undefined) {
        throw new BotdError(-1 /* Undefined */, 'document.documentElement is undefined');
    }
    const { documentElement } = document;
    if (typeof documentElement.getAttributeNames !== 'function') {
        throw new BotdError(104 /* NotFunction */, 'document.documentElement.getAttributeNames is not a function');
    }
    return documentElement.getAttributeNames();
}

function getWindowClose() {
    if (window.close === undefined) {
        throw new BotdError(-1 /* Undefined */, 'window.close is undefined');
    }
    return window.close.toString();
}

function getWindowExternal() {
    if (window.external === undefined) {
        throw new BotdError(-1 /* Undefined */, 'window.external is undefined');
    }
    const { external } = window;
    if (typeof external.toString !== 'function') {
        throw new BotdError(104 /* NotFunction */, 'window.external.toString is not a function');
    }
    return external.toString();
}

function getLanguages() {
    return Object.assign([], navigator.languages);
}

function getMimeTypesLength() {
    if (navigator.mimeTypes === undefined) {
        throw new BotdError(-1 /* Undefined */, 'navigator.mimeTypes is undefined');
    }
    return navigator.mimeTypes.length;
}

function areMimeTypesConsistent() {
    return __awaiter(this, void 0, void 0, function* () {
        if (navigator.mimeTypes === undefined) {
            return false;
        }
        const { mimeTypes } = navigator;
        let isConsistent = Object.getPrototypeOf(mimeTypes) === MimeTypeArray.prototype;
        for (let i = 0; i < mimeTypes.length; i++) {
            isConsistent && (isConsistent = Object.getPrototypeOf(mimeTypes[i]) === MimeType.prototype);
        }
        return isConsistent;
    });
}

function requiredAccelerometerPermission() {
    return typeof DeviceMotionEvent !== 'undefined' && typeof DeviceMotionEvent.requestPermission === 'function';
}

function getTimestamp() {
    return Date.now();
}

function getBackdropFilter() {
    return CSS.supports('backdrop-filter', 'blur(2px)');
}

function getASTCProfiles() {
    const canvasElement = document.createElement('canvas');
    if (typeof canvasElement.getContext !== 'function') {
        throw new BotdError(104 /* NotFunction */, 'HTMLCanvasElement.getContext is not a function');
    }
    const webGLContext = canvasElement.getContext('webgl');
    if (webGLContext == null) {
        throw new BotdError(101 /* Null */, 'WebGLRenderingContext is null');
    }
    else {
        const astcExtension = webGLContext.getExtension('WEBGL_compressed_texture_astc');
        if (astcExtension === null)
            return '';
        if (typeof astcExtension.getSupportedProfiles !== 'function') {
            throw new BotdError(104 /* NotFunction */, 'WEBGL_compressed_texture_astc.getSupportedProfiles is not a function');
        }
        return astcExtension.getSupportedProfiles().toString();
    }
}

function getHairlines() {
    if (window.devicePixelRatio === undefined) {
        throw new BotdError(-1 /* Undefined */, 'window.devicePixelRatio is undefined');
    }
    const el = document.createElement('div');
    el.style.border = '.5px dotted transparent';
    document.body.appendChild(el);
    const offsetHeight = el.offsetHeight;
    document.body.removeChild(el);
    return [window.devicePixelRatio, offsetHeight];
}

function isHiDPI() {
    if (typeof window.matchMedia !== 'function') {
        throw new BotdError(104 /* NotFunction */, 'window.matchMedia is not a function');
    }
    const query = '(-webkit-min-device-pixel-ratio: 2), (min-device-pixel-ratio: 2), (min-resolution: 192dpi)';
    const mql = window.matchMedia(query);
    if (mql.matches === undefined) {
        throw new BotdError(-1 /* Undefined */, 'MediaQueryList.matches is undefined');
    }
    return mql.matches;
}

function isDarkTheme() {
    if (typeof window.matchMedia !== 'function') {
        throw new BotdError(104 /* NotFunction */, 'window.matchMedia is not a function');
    }
    const query = '(prefers-color-scheme: dark)';
    const mql = window.matchMedia(query);
    if (mql.matches === undefined) {
        throw new BotdError(-1 /* Undefined */, 'MediaQueryList.matches is undefined');
    }
    return mql.matches;
}

function getSABByteLength() {
    if (typeof window.SharedArrayBuffer !== 'function') {
        throw new BotdError(104 /* NotFunction */, 'window.SharedArrayBuffer is not a function');
    }
    const singleElementSAB = new window.SharedArrayBuffer(1);
    if (singleElementSAB.byteLength === undefined) {
        throw new BotdError(-1 /* Undefined */, 'SharedArrayBuffer.byteLength is undefined');
    }
    return singleElementSAB.byteLength;
}

function handleSource(sourceFunction) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            return { state: 1 /* Success */, value: yield sourceFunction() };
        }
        catch (e) {
            if (e instanceof BotdError) {
                return { state: e.state, value: e.toString() };
            }
            else {
                return { state: -100 /* Unexpected */, value: e.toString() };
            }
        }
    });
}
function handleAll(sources) {
    return __awaiter(this, void 0, void 0, function* () {
        const components = {};
        for (const value of sources) {
            const index = sources.indexOf(value);
            const name = `s${index + 1}`;
            components[name] = yield handleSource(value);
        }
        return components;
    });
}
/**
 * Collects all the components from the browser.
 *
 * @returns {Promise<ComponentDict>} A promise to the collected components.
 */
function collect() {
    return __awaiter(this, void 0, void 0, function* () {
        return handleAll([
            getUserAgent,
            hasUserAgentData,
            getAppVersion,
            getRTT,
            getWindowOuterSize,
            arePermissionsInconsistent,
            getWebGL,
            getScreen,
            getDeviceMemory,
            isBigEndian,
            getHardwareConcurrency,
            getInstallTrigger,
            getWebDriver,
            getEvalLength,
            getPluginsLength,
            arePluginsConsistent,
            getErrorTrace,
            getErrorFF,
            getOSCPU,
            getPlatform,
            getProductSub,
            getVendor,
            getFrequency,
            getWindowProperties,
            getDocumentProperties,
            getNavigatorProperties,
            getTouchPoints,
            getSourceBufferType,
            getDocumentElementKeys,
            getWindowClose,
            getWindowExternal,
            getLanguages,
            getMimeTypesLength,
            areMimeTypesConsistent,
            requiredAccelerometerPermission,
            getTimestamp,
            getBackdropFilter,
            getASTCProfiles,
            getHairlines,
            isHiDPI,
            isDarkTheme,
            getSABByteLength, // s42
        ]);
    });
}

var version = "0.1.24";

// DEFLATE is a complex format; to read this code, you should probably check the RFC first:

// aliases for shorter compressed code (most minifers don't do this)
var u8 = Uint8Array, u16 = Uint16Array, u32 = Uint32Array;
// fixed length extra bits
var fleb = new u8([0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 2, 2, 2, 2, 3, 3, 3, 3, 4, 4, 4, 4, 5, 5, 5, 5, 0, /* unused */ 0, 0, /* impossible */ 0]);
// fixed distance extra bits
// see fleb note
var fdeb = new u8([0, 0, 0, 0, 1, 1, 2, 2, 3, 3, 4, 4, 5, 5, 6, 6, 7, 7, 8, 8, 9, 9, 10, 10, 11, 11, 12, 12, 13, 13, /* unused */ 0, 0]);
// code length index map
var clim = new u8([16, 17, 18, 0, 8, 7, 9, 6, 10, 5, 11, 4, 12, 3, 13, 2, 14, 1, 15]);
// get base, reverse index map from extra bits
var freb = function (eb, start) {
    var b = new u16(31);
    for (var i = 0; i < 31; ++i) {
        b[i] = start += 1 << eb[i - 1];
    }
    // numbers here are at max 18 bits
    var r = new u32(b[30]);
    for (var i = 1; i < 30; ++i) {
        for (var j = b[i]; j < b[i + 1]; ++j) {
            r[j] = ((j - b[i]) << 5) | i;
        }
    }
    return [b, r];
};
var _a = freb(fleb, 2), fl = _a[0], revfl = _a[1];
// we can ignore the fact that the other numbers are wrong; they never happen anyway
fl[28] = 258, revfl[258] = 28;
var _b = freb(fdeb, 0), revfd = _b[1];
// map of value to reverse (assuming 16 bits)
var rev = new u16(32768);
for (var i = 0; i < 32768; ++i) {
    // reverse table algorithm from SO
    var x = ((i & 0xAAAA) >>> 1) | ((i & 0x5555) << 1);
    x = ((x & 0xCCCC) >>> 2) | ((x & 0x3333) << 2);
    x = ((x & 0xF0F0) >>> 4) | ((x & 0x0F0F) << 4);
    rev[i] = (((x & 0xFF00) >>> 8) | ((x & 0x00FF) << 8)) >>> 1;
}
// create huffman tree from u8 "map": index -> code length for code index
// mb (max bits) must be at most 15
// TODO: optimize/split up?
var hMap = (function (cd, mb, r) {
    var s = cd.length;
    // index
    var i = 0;
    // u16 "map": index -> # of codes with bit length = index
    var l = new u16(mb);
    // length of cd must be 288 (total # of codes)
    for (; i < s; ++i) {
        if (cd[i])
            ++l[cd[i] - 1];
    }
    // u16 "map": index -> minimum code for bit length = index
    var le = new u16(mb);
    for (i = 0; i < mb; ++i) {
        le[i] = (le[i - 1] + l[i - 1]) << 1;
    }
    var co;
    if (r) {
        // u16 "map": index -> number of actual bits, symbol for code
        co = new u16(1 << mb);
        // bits to remove for reverser
        var rvb = 15 - mb;
        for (i = 0; i < s; ++i) {
            // ignore 0 lengths
            if (cd[i]) {
                // num encoding both symbol and bits read
                var sv = (i << 4) | cd[i];
                // free bits
                var r_1 = mb - cd[i];
                // start value
                var v = le[cd[i] - 1]++ << r_1;
                // m is end value
                for (var m = v | ((1 << r_1) - 1); v <= m; ++v) {
                    // every 16 bit value starting with the code yields the same result
                    co[rev[v] >>> rvb] = sv;
                }
            }
        }
    }
    else {
        co = new u16(s);
        for (i = 0; i < s; ++i) {
            if (cd[i]) {
                co[i] = rev[le[cd[i] - 1]++] >>> (15 - cd[i]);
            }
        }
    }
    return co;
});
// fixed length tree
var flt = new u8(288);
for (var i = 0; i < 144; ++i)
    flt[i] = 8;
for (var i = 144; i < 256; ++i)
    flt[i] = 9;
for (var i = 256; i < 280; ++i)
    flt[i] = 7;
for (var i = 280; i < 288; ++i)
    flt[i] = 8;
// fixed distance tree
var fdt = new u8(32);
for (var i = 0; i < 32; ++i)
    fdt[i] = 5;
// fixed length map
var flm = /*#__PURE__*/ hMap(flt, 9, 0);
// fixed distance map
var fdm = /*#__PURE__*/ hMap(fdt, 5, 0);
// get end of byte
var shft = function (p) { return ((p + 7) / 8) | 0; };
// typed array slice - allows garbage collector to free original reference,
// while being more compatible than .slice
var slc = function (v, s, e) {
    if (s == null || s < 0)
        s = 0;
    if (e == null || e > v.length)
        e = v.length;
    // can't use .constructor in case user-supplied
    var n = new (v.BYTES_PER_ELEMENT == 2 ? u16 : v.BYTES_PER_ELEMENT == 4 ? u32 : u8)(e - s);
    n.set(v.subarray(s, e));
    return n;
};
// starting at p, write the minimum number of bits that can hold v to d
var wbits = function (d, p, v) {
    v <<= p & 7;
    var o = (p / 8) | 0;
    d[o] |= v;
    d[o + 1] |= v >>> 8;
};
// starting at p, write the minimum number of bits (>8) that can hold v to d
var wbits16 = function (d, p, v) {
    v <<= p & 7;
    var o = (p / 8) | 0;
    d[o] |= v;
    d[o + 1] |= v >>> 8;
    d[o + 2] |= v >>> 16;
};
// creates code lengths from a frequency table
var hTree = function (d, mb) {
    // Need extra info to make a tree
    var t = [];
    for (var i = 0; i < d.length; ++i) {
        if (d[i])
            t.push({ s: i, f: d[i] });
    }
    var s = t.length;
    var t2 = t.slice();
    if (!s)
        return [et, 0];
    if (s == 1) {
        var v = new u8(t[0].s + 1);
        v[t[0].s] = 1;
        return [v, 1];
    }
    t.sort(function (a, b) { return a.f - b.f; });
    // after i2 reaches last ind, will be stopped
    // freq must be greater than largest possible number of symbols
    t.push({ s: -1, f: 25001 });
    var l = t[0], r = t[1], i0 = 0, i1 = 1, i2 = 2;
    t[0] = { s: -1, f: l.f + r.f, l: l, r: r };
    // efficient algorithm from UZIP.js
    // i0 is lookbehind, i2 is lookahead - after processing two low-freq
    // symbols that combined have high freq, will start processing i2 (high-freq,
    // non-composite) symbols instead
    // see https://reddit.com/r/photopea/comments/ikekht/uzipjs_questions/
    while (i1 != s - 1) {
        l = t[t[i0].f < t[i2].f ? i0++ : i2++];
        r = t[i0 != i1 && t[i0].f < t[i2].f ? i0++ : i2++];
        t[i1++] = { s: -1, f: l.f + r.f, l: l, r: r };
    }
    var maxSym = t2[0].s;
    for (var i = 1; i < s; ++i) {
        if (t2[i].s > maxSym)
            maxSym = t2[i].s;
    }
    // code lengths
    var tr = new u16(maxSym + 1);
    // max bits in tree
    var mbt = ln(t[i1 - 1], tr, 0);
    if (mbt > mb) {
        // more algorithms from UZIP.js
        // TODO: find out how this code works (debt)
        //  ind    debt
        var i = 0, dt = 0;
        //    left            cost
        var lft = mbt - mb, cst = 1 << lft;
        t2.sort(function (a, b) { return tr[b.s] - tr[a.s] || a.f - b.f; });
        for (; i < s; ++i) {
            var i2_1 = t2[i].s;
            if (tr[i2_1] > mb) {
                dt += cst - (1 << (mbt - tr[i2_1]));
                tr[i2_1] = mb;
            }
            else
                break;
        }
        dt >>>= lft;
        while (dt > 0) {
            var i2_2 = t2[i].s;
            if (tr[i2_2] < mb)
                dt -= 1 << (mb - tr[i2_2]++ - 1);
            else
                ++i;
        }
        for (; i >= 0 && dt; --i) {
            var i2_3 = t2[i].s;
            if (tr[i2_3] == mb) {
                --tr[i2_3];
                ++dt;
            }
        }
        mbt = mb;
    }
    return [new u8(tr), mbt];
};
// get the max length and assign length codes
var ln = function (n, l, d) {
    return n.s == -1
        ? Math.max(ln(n.l, l, d + 1), ln(n.r, l, d + 1))
        : (l[n.s] = d);
};
// length codes generation
var lc = function (c) {
    var s = c.length;
    // Note that the semicolon was intentional
    while (s && !c[--s])
        ;
    var cl = new u16(++s);
    //  ind      num         streak
    var cli = 0, cln = c[0], cls = 1;
    var w = function (v) { cl[cli++] = v; };
    for (var i = 1; i <= s; ++i) {
        if (c[i] == cln && i != s)
            ++cls;
        else {
            if (!cln && cls > 2) {
                for (; cls > 138; cls -= 138)
                    w(32754);
                if (cls > 2) {
                    w(cls > 10 ? ((cls - 11) << 5) | 28690 : ((cls - 3) << 5) | 12305);
                    cls = 0;
                }
            }
            else if (cls > 3) {
                w(cln), --cls;
                for (; cls > 6; cls -= 6)
                    w(8304);
                if (cls > 2)
                    w(((cls - 3) << 5) | 8208), cls = 0;
            }
            while (cls--)
                w(cln);
            cls = 1;
            cln = c[i];
        }
    }
    return [cl.subarray(0, cli), s];
};
// calculate the length of output from tree, code lengths
var clen = function (cf, cl) {
    var l = 0;
    for (var i = 0; i < cl.length; ++i)
        l += cf[i] * cl[i];
    return l;
};
// writes a fixed block
// returns the new bit pos
var wfblk = function (out, pos, dat) {
    // no need to write 00 as type: TypedArray defaults to 0
    var s = dat.length;
    var o = shft(pos + 2);
    out[o] = s & 255;
    out[o + 1] = s >>> 8;
    out[o + 2] = out[o] ^ 255;
    out[o + 3] = out[o + 1] ^ 255;
    for (var i = 0; i < s; ++i)
        out[o + i + 4] = dat[i];
    return (o + 4 + s) * 8;
};
// writes a block
var wblk = function (dat, out, final, syms, lf, df, eb, li, bs, bl, p) {
    wbits(out, p++, final);
    ++lf[256];
    var _a = hTree(lf, 15), dlt = _a[0], mlb = _a[1];
    var _b = hTree(df, 15), ddt = _b[0], mdb = _b[1];
    var _c = lc(dlt), lclt = _c[0], nlc = _c[1];
    var _d = lc(ddt), lcdt = _d[0], ndc = _d[1];
    var lcfreq = new u16(19);
    for (var i = 0; i < lclt.length; ++i)
        lcfreq[lclt[i] & 31]++;
    for (var i = 0; i < lcdt.length; ++i)
        lcfreq[lcdt[i] & 31]++;
    var _e = hTree(lcfreq, 7), lct = _e[0], mlcb = _e[1];
    var nlcc = 19;
    for (; nlcc > 4 && !lct[clim[nlcc - 1]]; --nlcc)
        ;
    var flen = (bl + 5) << 3;
    var ftlen = clen(lf, flt) + clen(df, fdt) + eb;
    var dtlen = clen(lf, dlt) + clen(df, ddt) + eb + 14 + 3 * nlcc + clen(lcfreq, lct) + (2 * lcfreq[16] + 3 * lcfreq[17] + 7 * lcfreq[18]);
    if (flen <= ftlen && flen <= dtlen)
        return wfblk(out, p, dat.subarray(bs, bs + bl));
    var lm, ll, dm, dl;
    wbits(out, p, 1 + (dtlen < ftlen)), p += 2;
    if (dtlen < ftlen) {
        lm = hMap(dlt, mlb, 0), ll = dlt, dm = hMap(ddt, mdb, 0), dl = ddt;
        var llm = hMap(lct, mlcb, 0);
        wbits(out, p, nlc - 257);
        wbits(out, p + 5, ndc - 1);
        wbits(out, p + 10, nlcc - 4);
        p += 14;
        for (var i = 0; i < nlcc; ++i)
            wbits(out, p + 3 * i, lct[clim[i]]);
        p += 3 * nlcc;
        var lcts = [lclt, lcdt];
        for (var it = 0; it < 2; ++it) {
            var clct = lcts[it];
            for (var i = 0; i < clct.length; ++i) {
                var len = clct[i] & 31;
                wbits(out, p, llm[len]), p += lct[len];
                if (len > 15)
                    wbits(out, p, (clct[i] >>> 5) & 127), p += clct[i] >>> 12;
            }
        }
    }
    else {
        lm = flm, ll = flt, dm = fdm, dl = fdt;
    }
    for (var i = 0; i < li; ++i) {
        if (syms[i] > 255) {
            var len = (syms[i] >>> 18) & 31;
            wbits16(out, p, lm[len + 257]), p += ll[len + 257];
            if (len > 7)
                wbits(out, p, (syms[i] >>> 23) & 31), p += fleb[len];
            var dst = syms[i] & 31;
            wbits16(out, p, dm[dst]), p += dl[dst];
            if (dst > 3)
                wbits16(out, p, (syms[i] >>> 5) & 8191), p += fdeb[dst];
        }
        else {
            wbits16(out, p, lm[syms[i]]), p += ll[syms[i]];
        }
    }
    wbits16(out, p, lm[256]);
    return p + ll[256];
};
// deflate options (nice << 13) | chain
var deo = /*#__PURE__*/ new u32([65540, 131080, 131088, 131104, 262176, 1048704, 1048832, 2114560, 2117632]);
// empty
var et = /*#__PURE__*/ new u8(0);
// compresses data into a raw DEFLATE buffer
var dflt = function (dat, lvl, plvl, pre, post, lst) {
    var s = dat.length;
    var o = new u8(pre + s + 5 * (1 + Math.ceil(s / 7000)) + post);
    // writing to this writes to the output buffer
    var w = o.subarray(pre, o.length - post);
    var pos = 0;
    if (!lvl || s < 8) {
        for (var i = 0; i <= s; i += 65535) {
            // end
            var e = i + 65535;
            if (e >= s) {
                // write final block
                w[pos >> 3] = lst;
            }
            pos = wfblk(w, pos + 1, dat.subarray(i, e));
        }
    }
    else {
        var opt = deo[lvl - 1];
        var n = opt >>> 13, c = opt & 8191;
        var msk_1 = (1 << plvl) - 1;
        //    prev 2-byte val map    curr 2-byte val map
        var prev = new u16(32768), head = new u16(msk_1 + 1);
        var bs1_1 = Math.ceil(plvl / 3), bs2_1 = 2 * bs1_1;
        var hsh = function (i) { return (dat[i] ^ (dat[i + 1] << bs1_1) ^ (dat[i + 2] << bs2_1)) & msk_1; };
        // 24576 is an arbitrary number of maximum symbols per block
        // 424 buffer for last block
        var syms = new u32(25000);
        // length/literal freq   distance freq
        var lf = new u16(288), df = new u16(32);
        //  l/lcnt  exbits  index  l/lind  waitdx  bitpos
        var lc_1 = 0, eb = 0, i = 0, li = 0, wi = 0, bs = 0;
        for (; i < s; ++i) {
            // hash value
            // deopt when i > s - 3 - at end, deopt acceptable
            var hv = hsh(i);
            // index mod 32768    previous index mod
            var imod = i & 32767, pimod = head[hv];
            prev[imod] = pimod;
            head[hv] = imod;
            // We always should modify head and prev, but only add symbols if
            // this data is not yet processed ("wait" for wait index)
            if (wi <= i) {
                // bytes remaining
                var rem = s - i;
                if ((lc_1 > 7000 || li > 24576) && rem > 423) {
                    pos = wblk(dat, w, 0, syms, lf, df, eb, li, bs, i - bs, pos);
                    li = lc_1 = eb = 0, bs = i;
                    for (var j = 0; j < 286; ++j)
                        lf[j] = 0;
                    for (var j = 0; j < 30; ++j)
                        df[j] = 0;
                }
                //  len    dist   chain
                var l = 2, d = 0, ch_1 = c, dif = (imod - pimod) & 32767;
                if (rem > 2 && hv == hsh(i - dif)) {
                    var maxn = Math.min(n, rem) - 1;
                    var maxd = Math.min(32767, i);
                    // max possible length
                    // not capped at dif because decompressors implement "rolling" index population
                    var ml = Math.min(258, rem);
                    while (dif <= maxd && --ch_1 && imod != pimod) {
                        if (dat[i + l] == dat[i + l - dif]) {
                            var nl = 0;
                            for (; nl < ml && dat[i + nl] == dat[i + nl - dif]; ++nl)
                                ;
                            if (nl > l) {
                                l = nl, d = dif;
                                // break out early when we reach "nice" (we are satisfied enough)
                                if (nl > maxn)
                                    break;
                                // now, find the rarest 2-byte sequence within this
                                // length of literals and search for that instead.
                                // Much faster than just using the start
                                var mmd = Math.min(dif, nl - 2);
                                var md = 0;
                                for (var j = 0; j < mmd; ++j) {
                                    var ti = (i - dif + j + 32768) & 32767;
                                    var pti = prev[ti];
                                    var cd = (ti - pti + 32768) & 32767;
                                    if (cd > md)
                                        md = cd, pimod = ti;
                                }
                            }
                        }
                        // check the previous match
                        imod = pimod, pimod = prev[imod];
                        dif += (imod - pimod + 32768) & 32767;
                    }
                }
                // d will be nonzero only when a match was found
                if (d) {
                    // store both dist and len data in one Uint32
                    // Make sure this is recognized as a len/dist with 28th bit (2^28)
                    syms[li++] = 268435456 | (revfl[l] << 18) | revfd[d];
                    var lin = revfl[l] & 31, din = revfd[d] & 31;
                    eb += fleb[lin] + fdeb[din];
                    ++lf[257 + lin];
                    ++df[din];
                    wi = i + l;
                    ++lc_1;
                }
                else {
                    syms[li++] = dat[i];
                    ++lf[dat[i]];
                }
            }
        }
        pos = wblk(dat, w, lst, syms, lf, df, eb, li, bs, i - bs, pos);
        // this is the easiest way to avoid needing to maintain state
        if (!lst && pos & 7)
            pos = wfblk(w, pos + 1, et);
    }
    return slc(o, 0, pre + shft(pos) + post);
};
// CRC32 table
var crct = /*#__PURE__*/ (function () {
    var t = new Int32Array(256);
    for (var i = 0; i < 256; ++i) {
        var c = i, k = 9;
        while (--k)
            c = ((c & 1) && -306674912) ^ (c >>> 1);
        t[i] = c;
    }
    return t;
})();
// CRC32
var crc = function () {
    var c = -1;
    return {
        p: function (d) {
            // closures have awful performance
            var cr = c;
            for (var i = 0; i < d.length; ++i)
                cr = crct[(cr & 255) ^ d[i]] ^ (cr >>> 8);
            c = cr;
        },
        d: function () { return ~c; }
    };
};
// deflate with opts
var dopt = function (dat, opt, pre, post, st) {
    return dflt(dat, opt.level == null ? 6 : opt.level, opt.mem == null ? Math.ceil(Math.max(8, Math.min(13, Math.log(dat.length))) * 1.5) : (12 + opt.mem), pre, post, !st);
};
// write bytes
var wbytes = function (d, b, v) {
    for (; v; ++b)
        d[b] = v, v >>>= 8;
};
// gzip header
var gzh = function (c, o) {
    var fn = o.filename;
    c[0] = 31, c[1] = 139, c[2] = 8, c[8] = o.level < 2 ? 4 : o.level == 9 ? 2 : 0, c[9] = 3; // assume Unix
    if (o.mtime != 0)
        wbytes(c, 4, Math.floor(new Date(o.mtime || Date.now()) / 1000));
    if (fn) {
        c[3] = 8;
        for (var i = 0; i <= fn.length; ++i)
            c[i + 10] = fn.charCodeAt(i);
    }
};
// gzip header length
var gzhl = function (o) { return 10 + ((o.filename && (o.filename.length + 1)) || 0); };
/**
 * Compresses data with GZIP
 * @param data The data to compress
 * @param opts The compression options
 * @returns The gzipped version of the data
 */
function gzipSync(data, opts) {
    if (!opts)
        opts = {};
    var c = crc(), l = data.length;
    c.p(data);
    var d = dopt(data, opts, gzhl(opts), 8), s = d.length;
    return gzh(d, opts), wbytes(d, s - 8, c.d()), wbytes(d, s - 4, l), d;
}
// text decoder
var td = typeof TextDecoder != 'undefined' && /*#__PURE__*/ new TextDecoder();
// text decoder stream
var tds = 0;
try {
    td.decode(et, { stream: true });
    tds = 1;
}
catch (e) { }

function getUTF8Bytes(text) {
    // Benchmark: https://jsbench.me/b6klaaxgwq/1
    // If you want to just count bytes, see solutions at https://jsbench.me/ehklab415e/1
    if (typeof TextEncoder === 'function') {
        return new TextEncoder().encode(text); // From https://stackoverflow.com/a/11411402/1118709
    }
    // From https://stackoverflow.com/a/18722848/1118709
    const binaryText = unescape(encodeURI(text));
    const bytes = new Uint8Array(binaryText.length);
    for (let i = 0; i < binaryText.length; ++i) {
        bytes[i] = binaryText.charCodeAt(i);
    }
    return bytes;
}
function parseUTF8Bytes(data) {
    if (typeof TextDecoder === 'function') {
        const decoded = new TextDecoder().decode(data); // From https://stackoverflow.com/a/11411402/1118709
        if (decoded) {
            return decoded; // The decoder always returns empty string in old Blink (e.g. Android 5.1 Browser)
        }
    }
    // From https://stackoverflow.com/a/18722848/111870
    const bytes = bufferSourceToBytes(data);
    return decodeURIComponent(escape(String.fromCharCode.apply(null, bytes)));
}
function bufferSourceToBytes(source) {
    if (source instanceof ArrayBuffer) {
        return new Uint8Array(source);
    }
    return new Uint8Array(source.buffer, source.byteOffset, source.byteLength);
}
class CompressedXorWithIndexObfuscation {
    constructor() {
        this.type = 2 /* CompressedXorWithIndex */;
        this.paddingSize = 0;
    }
    obfuscate(payload) {
        const payloadBytes = getUTF8Bytes(JSON.stringify(payload));
        const compressed = gzipSync(payloadBytes);
        const resultSize = 2 + compressed.length; // Type + padding size + padding + payload
        const result = new ArrayBuffer(resultSize);
        const resultBytes = new Uint8Array(result);
        let resultOffset = 0;
        resultBytes[resultOffset++] = this.type;
        resultBytes[resultOffset++] = this.paddingSize;
        // No padding
        for (let i = 0; i < compressed.length; ++i) {
            resultBytes[resultOffset++] = compressed[i] ^ i;
        }
        return resultBytes;
    }
    deobfuscate(encoded) {
        const encodedBytes = bufferSourceToBytes(encoded);
        const type = encodedBytes[0];
        const paddingSize = encodedBytes[1];
        if (type !== this.type || paddingSize !== this.paddingSize) {
            throw new BotdError(105 /* ObfuscationError */, 'Wrong type or padding size');
        }
        const headerOffset = 2; // Type and padding size
        const payloadBytes = new Uint8Array(encodedBytes.length - headerOffset);
        for (let i = 0; i < payloadBytes.length; ++i) {
            payloadBytes[i] = encodedBytes[i + headerOffset] ^ i;
        }
        return JSON.parse(parseUTF8Bytes(payloadBytes));
    }
}

function setCookie(name, value) {
    value = encodeURIComponent(value);
    document.cookie = `${name}=${value};SameSite=None;Secure`;
}
/**
 * Class representing a bot detector.
 *
 * @class
 * @implements {BotDetectorInterface}
 */
class BotDetector {
    constructor(options) {
        this.tag = '';
        this.endpoint = options.endpoint == undefined ? 'https://botd.fpapi.io/api/v1/' : options.endpoint;
        this.endpoint += this.endpoint.endsWith('/') ? '' : '/';
        if (this.endpoint.indexOf('://') === -1) {
            this.endpoint = new URL(this.endpoint, document.baseURI).href;
        }
        // TODO: Get rid of token
        const token = options.token == undefined ? '' : options.token;
        const publicKey = options.publicKey == undefined ? '' : options.publicKey;
        if (publicKey === '' && token === '') {
            throw BotDetector.createError("publicKeyRequired" /* PublicKeyRequired */, 'publicKey required');
        }
        this.publicKey = publicKey === '' ? token : publicKey;
        this.integration = options.mode === 'integration';
        this.mode = options.mode == undefined ? 'requestId' : this.integration ? 'requestId' : options.mode;
        this.obfuscator = new CompressedXorWithIndexObfuscation();
        this.obfuscationMode =
            options.obfuscationMode == undefined ? (this.integration ? 'requestOnly' : 'all') : options.obfuscationMode;
    }
    /**
     * @inheritdoc
     */
    collect() {
        return __awaiter(this, void 0, void 0, function* () {
            const timestamp = Date.now();
            this.components = yield collect();
            this.performance = Date.now() - timestamp;
            return this.components;
        });
    }
    static throwIfError(response) {
        if ('error' in response) {
            throw response;
        }
    }
    static createError(code, msg) {
        return {
            error: {
                code: code,
                message: msg,
            },
        };
    }
    createRequestBody() {
        return {
            mode: this.mode,
            performance: this.performance,
            signals: this.components,
            publicKey: this.publicKey,
            tag: this.tag,
        };
    }
    /**
     * @inheritdoc
     */
    detect(options = { tag: '' }) {
        return __awaiter(this, void 0, void 0, function* () {
            this.tag = options.tag;
            try {
                const credentials = this.integration ? 'include' : undefined;
                const url = new URL(this.endpoint);
                url.pathname += 'detect';
                url.searchParams.append('version', version);
                url.search += this.obfuscationMode !== 'all' ? '&deobfuscate' : '';
                const body = this.obfuscationMode === 'none'
                    ? JSON.stringify(this.createRequestBody())
                    : this.obfuscator.obfuscate(this.createRequestBody());
                const response = yield fetch(url.href, {
                    method: 'POST',
                    headers: { 'Content-Type': 'text/plain' },
                    body: body,
                    credentials: credentials,
                });
                const responseJSON = this.obfuscationMode !== 'all'
                    ? yield response.json()
                    : this.obfuscator.deobfuscate(yield response.arrayBuffer());
                BotDetector.throwIfError(responseJSON);
                if ('requestId' in responseJSON && !this.integration) {
                    setCookie('botd-request-id', responseJSON['requestId']);
                }
                return responseJSON;
            }
            catch (err) {
                BotDetector.throwIfError(err);
                throw BotDetector.createError("botdFailed" /* BotdFailed */, err.toString());
            }
        });
    }
}

/**
 * Builds an instance of the BotDetector. It's recommended to call it as early as possible, ideally during application startup.
 *
 * @param {InitOptions} options Configuration options.
 * @returns {Promise<BotDetectorInterface>} A promise to the instance of the bot detector.
 */
function load(options) {
    return __awaiter(this, void 0, void 0, function* () {
        const detector = new BotDetector(options);
        yield detector.collect();
        return detector;
    });
}
var index = { load };

exports.default = index;
exports.load = load;
